@@ -3,7 +3,7 @@ from __future__ import annotations
 import abc
 import asyncio
 import collections
-from collections.abc import Coroutine
+from collections.abc import AsyncGenerator, Coroutine
 import contextlib
 from datetime import datetime, timezone
 import errno
@@ -16,11 +16,15 @@ import typing
 from typing import Any, TypeVar
 import warnings
 
+from zigpy.backports.contextlib import nullcontext
+
 if sys.version_info[:2] < (3, 11):
     from async_timeout import timeout as asyncio_timeout  # pragma: no cover
 else:
     from asyncio import timeout as asyncio_timeout  # pragma: no cover
 
+import contextvars
+
 import zigpy.appdb
 import zigpy.backups
 import zigpy.config as conf
@@ -70,7 +74,6 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         self._config = self.SCHEMA(config)
         self._dblistener = None
         self._groups = zigpy.group.Groups(self)
-        self._listeners = {}
         self._send_sequence = 0
         self._tasks: set[asyncio.Future[Any]] = set()
 
@@ -85,10 +88,15 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         self.topology: zigpy.topology.Topology = zigpy.topology.Topology(self)
 
         self._req_listeners: collections.defaultdict[
-            zigpy.device.Device,
+            zigpy.device.Device | zigpy.listeners.Singleton,
             collections.deque[zigpy.listeners.BaseRequestListener],
         ] = collections.defaultdict(lambda: collections.deque([]))
 
+        # Context variable for request priority context manager
+        self._packet_priority_var = contextvars.ContextVar(
+            "request_priority", default=t.PacketPriority.NORMAL
+        )
+
     def create_task(
         self, target: Coroutine[Any, Any, _R], name: str | None = None
     ) -> asyncio.Task[_R]:
@@ -101,6 +109,16 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         task.add_done_callback(self._tasks.remove)
         return task
 
+    @contextlib.asynccontextmanager
+    async def request_priority(self, priority: int) -> AsyncGenerator[None]:
+        """Context manager to set the request priority for the duration of the context."""
+        token = self._packet_priority_var.set(priority)
+
+        try:
+            yield
+        finally:
+            self._packet_priority_var.reset(token)
+
     async def _load_db(self) -> None:
         """Restore save state."""
         database_file = self.config[conf.CONF_DATABASE]
@@ -417,10 +435,16 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         if self._watchdog_task is not None:
             self._watchdog_task.cancel()
 
+        for task in self._tasks:
+            task.cancel()
+
         self.ota.stop_periodic_broadcasts()
         self.backups.stop_periodic_backups()
         self.topology.stop_periodic_scans()
 
+        for device in self.devices.values():
+            device.on_remove()
+
         try:
             await self.disconnect()
         except Exception:  # noqa: BLE001
@@ -741,11 +765,27 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
             await self.add_endpoint(endpoint)
 
     @contextlib.asynccontextmanager
-    async def _limit_concurrency(self, *, priority: int = t.PacketPriority.NORMAL):
+    async def _limit_concurrency(
+        self, *, priority: int | None = None
+    ) -> AsyncGenerator[None, None]:
         """Async context manager to limit global coordinator request concurrency."""
+        if priority is None:
+            priority = self._packet_priority_var.get()
 
         start_time = time.monotonic()
-        was_locked = self._concurrent_requests_semaphore.locked()
+        manager: contextlib.AbstractAsyncContextManager
+
+        if priority >= t.PacketPriority.CRITICAL:
+            LOGGER.debug(
+                "Critical priority request received (%s), skipping queue with %d requests",
+                priority,
+                self._concurrent_requests_semaphore.num_waiting,
+            )
+            manager = nullcontext()
+            was_locked = False
+        else:
+            manager = self._concurrent_requests_semaphore(priority=priority)
+            was_locked = self._concurrent_requests_semaphore.locked()
 
         if was_locked:
             LOGGER.debug(
@@ -754,7 +794,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
                 self._concurrent_requests_semaphore.num_waiting,
             )
 
-        async with self._concurrent_requests_semaphore(priority=priority):
+        async with manager:
             if was_locked:
                 LOGGER.debug(
                     "Previously delayed request is now running, delayed by %0.2fs",
@@ -832,22 +872,44 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         elif not expect_reply:
             tx_options |= t.TransmitOptions.ACK
 
-        await self.send_packet(
-            t.ZigbeePacket(
-                src=src,
-                src_ep=src_ep,
-                dst=dst,
-                dst_ep=dst_ep,
-                tsn=sequence,
-                profile_id=profile,
-                cluster_id=cluster,
-                data=t.SerializableBytes(data),
-                extended_timeout=extended_timeout,
-                source_route=source_route,
-                tx_options=tx_options,
-                priority=priority,
-            )
-        )
+        # Performing retries within zigpy allows us to reprioritize requests quickly
+        # without locking up for ~30s when communicating with end devices
+        max_attempts = self._config[conf.CONF_NWK_MAX_RETRIES] + 1
+
+        for attempt in range(max_attempts):
+            if attempt > 0:
+                tx_options |= t.TransmitOptions.FORCE_ROUTE_DISCOVERY
+
+            try:
+                await self.send_packet(
+                    t.ZigbeePacket(
+                        src=src,
+                        src_ep=src_ep,
+                        dst=dst,
+                        dst_ep=dst_ep,
+                        tsn=sequence,
+                        profile_id=profile,
+                        cluster_id=cluster,
+                        data=t.SerializableBytes(data),
+                        extended_timeout=extended_timeout,
+                        source_route=source_route,
+                        tx_options=tx_options,
+                        priority=priority,
+                    )
+                )
+                break
+            except Exception:
+                LOGGER.debug(
+                    "Failed to send packet, attempt %d of %d",
+                    attempt + 1,
+                    max_attempts,
+                    exc_info=True,
+                )
+
+                if attempt >= max_attempts - 1:
+                    raise
+
+                continue
 
         return (zigpy.zcl.foundation.Status.SUCCESS, "")
 
@@ -1045,13 +1107,17 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
             or device.all_endpoints_init
             or (
                 device.has_non_zdo_endpoints
-                and packet.cluster_id == zigpy.zcl.clusters.general.Basic.cluster_id
+                and packet.cluster_id
+                in (
+                    zigpy.zcl.clusters.general.Basic.cluster_id,
+                    zigpy.zcl.clusters.general.PollControl.cluster_id,
+                )
             )
         ):
             # Allow the following responses:
             #  - any ZDO
             #  - ZCL if endpoints are initialized
-            #  - ZCL from Basic packet.cluster_id if endpoints are initializing
+            #  - ZCL from Basic or PollControl clusters, if endpoints are initializing
 
             if not device.initializing:
                 device.schedule_initialize()
@@ -1128,6 +1194,32 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         else:
             raise ValueError(f"Invalid address: {address!r}")
 
+    def register_callback_listener(
+        self,
+        src: zigpy.device.Device | zigpy.listeners.ANY_DEVICE,
+        filters: list[zigpy.listeners.MatcherType],
+        callback: typing.Callable[
+            [
+                zigpy.zcl.foundation.ZCLHeader,
+                zigpy.zcl.foundation.CommandSchema,
+            ],
+            typing.Any,
+        ],
+    ) -> typing.Callable[[], None]:
+        listener = zigpy.listeners.CallbackListener(
+            matchers=tuple(filters),
+            callback=callback,
+        )
+
+        self._req_listeners[src].append(listener)
+
+        def cancel_callback() -> None:
+            """Remove the listener."""
+            if listener in self._req_listeners[src]:
+                self._req_listeners[src].remove(listener)
+
+        return cancel_callback
+
     @contextlib.contextmanager
     def callback_for_response(
         self,
@@ -1142,18 +1234,14 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         ],
     ) -> typing.Any:
         """Context manager to create a callback that is passed Zigbee responses."""
-
-        listener = zigpy.listeners.CallbackListener(
-            matchers=tuple(filters),
-            callback=callback,
+        cancel = self.register_callback_listener(
+            src=src, filters=filters, callback=callback
         )
 
-        self._req_listeners[src].append(listener)
-
         try:
             yield
         finally:
-            self._req_listeners[src].remove(listener)
+            cancel()
 
     @contextlib.contextmanager
     def wait_for_response(
@@ -1231,6 +1319,46 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
 
         raise NotImplementedError  # pragma: no cover
 
+    async def network_scan(
+        self, channels: t.Channels, duration_exp: int
+    ) -> AsyncGenerator[t.NetworkBeacon, None]:
+        """Scans for 802.15.4 networks with a specified duration exponent."""
+        async for network in self._network_scan(
+            channels=channels, duration_exp=duration_exp
+        ):
+            yield network
+
+    # @abc.abstractmethod
+    async def _network_scan(
+        self, channels: t.Channels, duration_exp: int
+    ) -> AsyncGenerator[t.NetworkBeacon, None]:
+        """Scans for 802.15.4 networks with a specified duration exponent."""
+        if False:
+            yield  # pragma: no cover
+
+    async def packet_capture(
+        self, channel: int
+    ) -> AsyncGenerator[t.CapturedPacket, None]:
+        """Packet capture on the specified channel."""
+        async for packet in self._packet_capture(channel=channel):
+            yield packet
+
+    # @abc.abstractmethod
+    async def _packet_capture(
+        self, channel: int
+    ) -> AsyncGenerator[t.CapturedPacket, None]:
+        """Packet capture on the specified channel, internal."""
+        if False:
+            yield  # pragma: no cover
+
+    async def packet_capture_change_channel(self, channel: int) -> None:
+        """Change the channel of an active packet capture."""
+        await self._packet_capture_change_channel(channel=channel)
+
+    # @abc.abstractmethod
+    async def _packet_capture_change_channel(self, channel: int) -> None:
+        """Change the channel of an active packet capture, internal."""
+
     async def permit(self, time_s: int = 60, node: t.EUI64 | str | None = None) -> None:
         """Permit joining on a specific node or all router nodes."""
         assert 0 <= time_s <= 254
