@@ -10,11 +10,10 @@ import inspect
 import logging
 import pathlib
 from types import FrameType
-import typing
 from typing import TYPE_CHECKING, Any, Callable
 
 import attrs
-from frozendict import deepfreeze, frozendict
+from frozendict import frozendict
 
 from zigpy.const import (
     SIG_ENDPOINTS,
@@ -34,6 +33,7 @@ from zigpy.quirks.v2.homeassistant.number import NumberDeviceClass
 from zigpy.quirks.v2.homeassistant.sensor import SensorDeviceClass, SensorStateClass
 import zigpy.types as t
 from zigpy.zcl import ClusterType
+from zigpy.zcl.clusters.general import Ota
 from zigpy.zdo import ZDO
 from zigpy.zdo.types import NodeDescriptor
 
@@ -89,21 +89,21 @@ class CustomDeviceV2(BaseCustomDevice):
         ] = collections.defaultdict(list)
 
         # endpoints need to be modified before clusters
-        for add_endpoint_meta in quirk_metadata.adds_endpoint_metadata:
-            add_endpoint_meta(self)
-
         for remove_endpoint_meta in quirk_metadata.removes_endpoint_metadata:
             remove_endpoint_meta(self)
 
+        for add_endpoint_meta in quirk_metadata.adds_endpoint_metadata:
+            add_endpoint_meta(self)
+
         for replace_endpoint_meta in quirk_metadata.replaces_endpoint_metadata:
             replace_endpoint_meta(self)
 
-        for add_meta in quirk_metadata.adds_metadata:
-            add_meta(self)
-
         for remove_meta in quirk_metadata.removes_metadata:
             remove_meta(self)
 
+        for add_meta in quirk_metadata.adds_metadata:
+            add_meta(self)
+
         for replace_meta in quirk_metadata.replaces_metadata:
             replace_meta(self)
 
@@ -166,8 +166,8 @@ class AddsMetadata:
     cluster: int | type[Cluster | CustomCluster] = attrs.field()
     endpoint_id: int = attrs.field(default=1)
     cluster_type: ClusterType = attrs.field(default=ClusterType.Server)
-    constant_attributes: frozendict[ZCLAttributeDef, typing.Any] = attrs.field(
-        factory=frozendict, converter=deepfreeze
+    constant_attributes: frozendict[ZCLAttributeDef, Any] = attrs.field(
+        factory=frozendict, converter=frozendict
     )
 
     def __call__(self, device: CustomDeviceV2) -> None:
@@ -285,13 +285,18 @@ class RemovesEndpointMetadata:
 class ReplacesEndpointMetadata:
     """Replaces metadata for replacing an endpoint on a device."""
 
-    remove: RemovesEndpointMetadata = attrs.field()
-    add: AddsEndpointMetadata = attrs.field()
+    endpoint_id: int = attrs.field()
+    profile_id: int = attrs.field()
+    device_type: int = attrs.field()
 
     def __call__(self, device: CustomDeviceV2) -> None:
         """Process the replace."""
-        self.remove(device)
-        self.add(device)
+        if self.endpoint_id in device.endpoints:
+            ep: Endpoint = device.endpoints[self.endpoint_id]
+        else:
+            ep = device.add_endpoint(self.endpoint_id)
+        ep.profile_id = self.profile_id
+        ep.device_type = self.device_type
 
 
 @attrs.define(frozen=True, kw_only=True, repr=True)
@@ -344,7 +349,7 @@ class ZCLSensorMetadata(EntityMetadata):
     """Metadata for exposed ZCL attribute based sensor entity."""
 
     attribute_name: str | None = attrs.field(default=None)
-    attribute_converter: typing.Callable[[Any], Any] | None = attrs.field(default=None)
+    attribute_converter: Callable[[Any], Any] | None = attrs.field(default=None)
     reporting_config: ReportingConfig | None = attrs.field(default=None)
     divisor: int | None = attrs.field(default=None)
     multiplier: int | None = attrs.field(default=None)
@@ -386,7 +391,7 @@ class BinarySensorMetadata(EntityMetadata):
     """Metadata for exposed binary sensor entity."""
 
     attribute_name: str = attrs.field()
-    attribute_converter: typing.Callable[[Any], Any] | None = attrs.field(default=None)
+    attribute_converter: Callable[[Any], Any] | None = attrs.field(default=None)
     reporting_config: ReportingConfig | None = attrs.field(default=None)
     device_class: BinarySensorDeviceClass | None = attrs.field(default=None)
 
@@ -448,7 +453,38 @@ class PreventDefaultEntityCreationMetadata:
     cluster_id: int | None = attrs.field()
     cluster_type: ClusterType | None = attrs.field()
     unique_id_suffix: str | None = attrs.field()
-    function: Callable[Any, bool] | None = attrs.field()
+    function: Callable[[Any], bool] | None = attrs.field()
+
+
+@attrs.define(frozen=True, kw_only=True, repr=True)
+class ChangedEntityMetadata:
+    """Metadata to change entity metadata for matching entities."""
+
+    endpoint_id: int | None = attrs.field()
+    cluster_id: int | None = attrs.field()
+    cluster_type: ClusterType | None = attrs.field()
+    unique_id_suffix: str | None = attrs.field()
+    function: Callable[[Any], bool] | None = attrs.field()
+    # Entity metadata changes
+    new_primary: bool | None = attrs.field(default=None)
+    new_unique_id: str | None = attrs.field(default=None)
+    new_translation_key: str | None = attrs.field(default=None)
+    new_device_class: (
+        BinarySensorDeviceClass | NumberDeviceClass | SensorDeviceClass | None
+    ) = attrs.field(default=None)
+    new_state_class: SensorStateClass | None = attrs.field(default=None)
+    new_entity_category: EntityType | None = attrs.field(default=None)
+    new_entity_registry_enabled_default: bool | None = attrs.field(default=None)
+    new_fallback_name: str | None = attrs.field(default=None)
+
+
+@attrs.define(frozen=True, kw_only=True, repr=True)
+class FirmwareVersionFilterMetadata:
+    """Metadata to only apply the quirk if the device's firmware version matches."""
+
+    min_version: int | None = attrs.field(default=None)
+    max_version: int | None = attrs.field(default=None)
+    allow_missing: bool = attrs.field(default=True)
 
 
 @attrs.define(frozen=True, kw_only=True, repr=True)
@@ -465,7 +501,9 @@ class QuirksV2RegistryEntry:
     disabled_default_entities: tuple[PreventDefaultEntityCreationMetadata] = (
         attrs.field(factory=tuple)
     )
+    changed_entity_metadata: tuple[ChangedEntityMetadata] = attrs.field(factory=tuple)
     filters: tuple[FilterType] = attrs.field(factory=tuple)
+    fw_version_filter: FirmwareVersionFilterMetadata | None = attrs.field(default=None)
     custom_device_class: type[CustomDeviceV2] | None = attrs.field(default=None)
     device_node_descriptor: NodeDescriptor | None = attrs.field(default=None)
     skip_device_configuration: bool = attrs.field(default=False)
@@ -492,11 +530,40 @@ class QuirksV2RegistryEntry:
     ] = attrs.field(factory=tuple)
     device_automation_triggers_metadata: frozendict[
         tuple[str, str], frozendict[str, str]
-    ] = attrs.field(factory=frozendict, converter=deepfreeze)
+    ] = attrs.field(
+        factory=frozendict,
+        converter=lambda d: frozendict({k: frozendict(v) for k, v in d.items()}),
+    )
 
     def matches_device(self, device: Device) -> bool:
         """Determine if this quirk should be applied to the passed in device."""
-        return all(_filter(device) for _filter in self.filters)
+        if not all(_filter(device) for _filter in self.filters):
+            return False
+
+        if self.fw_version_filter is not None:
+            try:
+                ota = device.find_cluster(
+                    cluster_id=Ota.cluster_id, cluster_type=ClusterType.Client
+                )
+            except ValueError:
+                return self.fw_version_filter.allow_missing
+
+            current_file_version = ota.get(Ota.AttributeDefs.current_file_version.id)
+
+            if current_file_version is None:
+                return self.fw_version_filter.allow_missing
+
+            if self.fw_version_filter.min_version is not None and (
+                current_file_version < self.fw_version_filter.min_version
+            ):
+                return False
+
+            if self.fw_version_filter.max_version is not None and (
+                current_file_version >= self.fw_version_filter.max_version
+            ):
+                return False
+
+        return True
 
     def create_device(self, device: Device) -> CustomDeviceV2:
         """Create the quirked device."""
@@ -527,7 +594,9 @@ class QuirkBuilder:
         self.friendly_name_metadata: FriendlyNameMetadata | None = None
         self.device_alerts: list[DeviceAlertMetadata] = []
         self.disabled_default_entities: list[PreventDefaultEntityCreationMetadata] = []
+        self.changed_entity_metadata: list[ChangedEntityMetadata] = []
         self.filters: list[FilterType] = []
+        self.fw_version_filter: FirmwareVersionFilterMetadata | None = None
         self.custom_device_class: type[CustomDeviceV2] | None = None
         self.device_node_descriptor: NodeDescriptor | None = None
         self.skip_device_configuration: bool = False
@@ -595,6 +664,25 @@ class QuirkBuilder:
         self.filters.append(filter_function)
         return self
 
+    def firmware_version_filter(
+        self,
+        min_version: int | None = None,
+        max_version: int | None = None,
+        allow_missing: bool = True,
+    ) -> QuirkBuilder:
+        """Add a firmware version filter and returns self.
+
+        The min_version and max_version are integers representing the firmware version,
+        minimum inclusive but maximum exclusive. If allow_missing is True, the filter
+        will pass if the device does not have a firmware version.
+        """
+        self.fw_version_filter = FirmwareVersionFilterMetadata(
+            min_version=min_version,
+            max_version=max_version,
+            allow_missing=allow_missing,
+        )
+        return self
+
     def device_class(self, custom_device_class: type[CustomDeviceV2]) -> QuirkBuilder:
         """Set the custom device class to be used in this quirk and returns self.
 
@@ -629,7 +717,7 @@ class QuirkBuilder:
         cluster: int | type[Cluster | CustomCluster],
         cluster_type: ClusterType = ClusterType.Server,
         endpoint_id: int = 1,
-        constant_attributes: dict[ZCLAttributeDef, typing.Any] | None = None,
+        constant_attributes: dict[ZCLAttributeDef, Any] | None = None,
     ) -> QuirkBuilder:
         """Add an AddsMetadata entry and returns self.
 
@@ -762,11 +850,9 @@ class QuirkBuilder:
         device_type: int = 0xFF,
     ) -> QuirkBuilder:
         """Add a ReplacesEndpointMetadata entry and return self."""
-        remove = RemovesEndpointMetadata(endpoint_id=endpoint_id)
-        add = AddsEndpointMetadata(
+        replace = ReplacesEndpointMetadata(
             endpoint_id=endpoint_id, profile_id=profile_id, device_type=device_type
         )
-        replace = ReplacesEndpointMetadata(remove=remove, add=add)
         self.replaces_endpoint_metadata.append(replace)
         return self
 
@@ -826,7 +912,7 @@ class QuirkBuilder:
         unit: str | None = None,
         initially_disabled: bool = False,
         attribute_initialized_from_cache: bool = True,
-        attribute_converter: typing.Callable[[Any], Any] | None = None,
+        attribute_converter: Callable[[Any], Any] | None = None,
         reporting_config: ReportingConfig | None = None,
         unique_id_suffix: str | None = None,
         translation_key: str | None = None,
@@ -972,7 +1058,7 @@ class QuirkBuilder:
         device_class: BinarySensorDeviceClass | None = None,
         initially_disabled: bool = False,
         attribute_initialized_from_cache: bool = True,
-        attribute_converter: typing.Callable[[Any], Any] | None = None,
+        attribute_converter: Callable[[Any], Any] | None = None,
         reporting_config: ReportingConfig | None = None,
         unique_id_suffix: str | None = None,
         translation_key: str | None = None,
@@ -1108,7 +1194,7 @@ class QuirkBuilder:
         cluster_id: int | None = None,
         cluster_type: ClusterType | None = None,
         unique_id_suffix: str | None = None,
-        function: Callable[Any, bool] | None = None,
+        function: Callable[[Any], bool] | None = None,
     ) -> QuirkBuilder:
         """Do not create default entities."""
         if cluster_id is not None and cluster_type is None:
@@ -1125,6 +1211,48 @@ class QuirkBuilder:
         )
         return self
 
+    def change_entity_metadata(
+        self,
+        *,
+        endpoint_id: int | None = None,
+        cluster_id: int | None = None,
+        cluster_type: ClusterType | None = None,
+        unique_id_suffix: str | None = None,
+        function: Callable[[Any], bool] | None = None,
+        new_primary: bool | None = None,
+        new_unique_id: str | None = None,
+        new_translation_key: str | None = None,
+        new_device_class: (
+            BinarySensorDeviceClass | NumberDeviceClass | SensorDeviceClass | None
+        ) = None,
+        new_state_class: SensorStateClass | None = None,
+        new_entity_category: EntityType | None = None,
+        new_entity_registry_enabled_default: bool | None = None,
+        new_fallback_name: str | None = None,
+    ) -> QuirkBuilder:
+        """Change entity metadata for matching entities."""
+        if cluster_id is not None and cluster_type is None:
+            cluster_type = ClusterType.Server
+
+        self.changed_entity_metadata.append(
+            ChangedEntityMetadata(
+                endpoint_id=endpoint_id,
+                cluster_id=cluster_id,
+                cluster_type=cluster_type,
+                unique_id_suffix=unique_id_suffix,
+                function=function,
+                new_primary=new_primary,
+                new_unique_id=new_unique_id,
+                new_translation_key=new_translation_key,
+                new_device_class=new_device_class,
+                new_state_class=new_state_class,
+                new_entity_category=new_entity_category,
+                new_entity_registry_enabled_default=new_entity_registry_enabled_default,
+                new_fallback_name=new_fallback_name,
+            ),
+        )
+        return self
+
     def add_to_registry(self) -> QuirksV2RegistryEntry:
         """Build the quirks v2 registry entry."""
         if not self.manufacturer_model_metadata:
@@ -1136,9 +1264,11 @@ class QuirkBuilder:
             friendly_name=self.friendly_name_metadata,
             device_alerts=tuple(self.device_alerts),
             disabled_default_entities=tuple(self.disabled_default_entities),
+            changed_entity_metadata=tuple(self.changed_entity_metadata),
             quirk_file=self.quirk_file,
             quirk_file_line=self.quirk_file_line,
             filters=tuple(self.filters),
+            fw_version_filter=self.fw_version_filter,
             custom_device_class=self.custom_device_class,
             device_node_descriptor=self.device_node_descriptor,
             skip_device_configuration=self.skip_device_configuration,
