@@ -11,7 +11,6 @@ from zigpy.zcl import Cluster, foundation
 from zigpy.zcl.foundation import (
     BaseAttributeDefs,
     BaseCommandDefs,
-    Direction,
     ZCLAttributeDef,
     ZCLCommandDef,
 )
@@ -279,9 +278,7 @@ class Basic(Cluster):
         reporting_status: Final = foundation.ZCL_REPORTING_STATUS_ATTR
 
     class ServerCommandDefs(BaseCommandDefs):
-        reset_fact_default: Final = ZCLCommandDef(
-            id=0x00, schema={}, direction=Direction.Client_to_Server
-        )
+        reset_fact_default: Final = ZCLCommandDef(id=0x00, schema={})
 
     def handle_read_attribute_zcl_version(self) -> t.uint8_t:
         return t.uint8_t(8)
@@ -582,24 +579,19 @@ class Identify(Cluster):
         identify: Final = ZCLCommandDef(
             id=0x00,
             schema={"identify_time": t.uint16_t},
-            direction=Direction.Client_to_Server,
-        )
-        identify_query: Final = ZCLCommandDef(
-            id=0x01, schema={}, direction=Direction.Client_to_Server
         )
+        identify_query: Final = ZCLCommandDef(id=0x01, schema={})
         # 0x02: ("ezmode_invoke", (t.bitmap8,), False),
         # 0x03: ("update_commission_state", (t.bitmap8,), False),
         trigger_effect: Final = ZCLCommandDef(
             id=0x40,
             schema={"effect_id": EffectIdentifier, "effect_variant": EffectVariant},
-            direction=Direction.Client_to_Server,
         )
 
     class ClientCommandDefs(BaseCommandDefs):
         identify_query_response: Final = ZCLCommandDef(
             id=0x00,
             schema={"timeout": t.uint16_t},
-            direction=Direction.Server_to_Client,
         )
 
 
@@ -628,33 +620,23 @@ class Groups(Cluster):
         add: Final = ZCLCommandDef(
             id=0x00,
             schema={"group_id": t.Group, "group_name": t.LimitedCharString(16)},
-            direction=Direction.Client_to_Server,
-        )
-        view: Final = ZCLCommandDef(
-            id=0x01, schema={"group_id": t.Group}, direction=Direction.Client_to_Server
         )
+        view: Final = ZCLCommandDef(id=0x01, schema={"group_id": t.Group})
         get_membership: Final = ZCLCommandDef(
             id=0x02,
             schema={"groups": t.LVList[t.Group]},
-            direction=Direction.Client_to_Server,
-        )
-        remove: Final = ZCLCommandDef(
-            id=0x03, schema={"group_id": t.Group}, direction=Direction.Client_to_Server
-        )
-        remove_all: Final = ZCLCommandDef(
-            id=0x04, schema={}, direction=Direction.Client_to_Server
         )
+        remove: Final = ZCLCommandDef(id=0x03, schema={"group_id": t.Group})
+        remove_all: Final = ZCLCommandDef(id=0x04, schema={})
         add_if_identifying: Final = ZCLCommandDef(
             id=0x05,
             schema={"group_id": t.Group, "group_name": t.LimitedCharString(16)},
-            direction=Direction.Client_to_Server,
         )
 
     class ClientCommandDefs(BaseCommandDefs):
         add_response: Final = ZCLCommandDef(
             id=0x00,
             schema={"status": foundation.Status, "group_id": t.Group},
-            direction=Direction.Server_to_Client,
         )
         view_response: Final = ZCLCommandDef(
             id=0x01,
@@ -663,17 +645,14 @@ class Groups(Cluster):
                 "group_id": t.Group,
                 "group_name": t.LimitedCharString(16),
             },
-            direction=Direction.Server_to_Client,
         )
         get_membership_response: Final = ZCLCommandDef(
             id=0x02,
             schema={"capacity": t.uint8_t, "groups": t.LVList[t.Group]},
-            direction=Direction.Server_to_Client,
         )
         remove_response: Final = ZCLCommandDef(
             id=0x03,
             schema={"status": foundation.Status, "group_id": t.Group},
-            direction=Direction.Server_to_Client,
         )
 
 
@@ -717,26 +696,20 @@ class Scenes(Cluster):
                 "transition_time": t.uint16_t,
                 "scene_name": t.LimitedCharString(16),
             },
-            direction=Direction.Client_to_Server,
         )
         # TODO: + extension field sets
         view: Final = ZCLCommandDef(
             id=0x01,
             schema={"group_id": t.Group, "scene_id": t.uint8_t},
-            direction=Direction.Client_to_Server,
         )
         remove: Final = ZCLCommandDef(
             id=0x02,
             schema={"group_id": t.Group, "scene_id": t.uint8_t},
-            direction=Direction.Client_to_Server,
-        )
-        remove_all: Final = ZCLCommandDef(
-            id=0x03, schema={"group_id": t.Group}, direction=Direction.Client_to_Server
         )
+        remove_all: Final = ZCLCommandDef(id=0x03, schema={"group_id": t.Group})
         store: Final = ZCLCommandDef(
             id=0x04,
             schema={"group_id": t.Group, "scene_id": t.uint8_t},
-            direction=Direction.Client_to_Server,
         )
         recall: Final = ZCLCommandDef(
             id=0x05,
@@ -745,10 +718,9 @@ class Scenes(Cluster):
                 "scene_id": t.uint8_t,
                 "transition_time?": t.uint16_t,
             },
-            direction=Direction.Client_to_Server,
         )
         get_scene_membership: Final = ZCLCommandDef(
-            id=0x06, schema={"group_id": t.Group}, direction=Direction.Client_to_Server
+            id=0x06, schema={"group_id": t.Group}
         )
         enhanced_add: Final = ZCLCommandDef(
             id=0x40,
@@ -758,12 +730,10 @@ class Scenes(Cluster):
                 "transition_time": t.uint16_t,
                 "scene_name": t.LimitedCharString(16),
             },
-            direction=Direction.Client_to_Server,
         )
         enhanced_view: Final = ZCLCommandDef(
             id=0x41,
             schema={"group_id": t.Group, "scene_id": t.uint8_t},
-            direction=Direction.Client_to_Server,
         )
         copy: Final = ZCLCommandDef(
             id=0x42,
@@ -774,7 +744,6 @@ class Scenes(Cluster):
                 "group_id_to": t.uint16_t,
                 "scene_id_to": t.uint8_t,
             },
-            direction=Direction.Client_to_Server,
         )
 
     class ClientCommandDefs(BaseCommandDefs):
@@ -785,7 +754,6 @@ class Scenes(Cluster):
                 "group_id": t.Group,
                 "scene_id": t.uint8_t,
             },
-            direction=Direction.Server_to_Client,
         )
         view_response: Final = ZCLCommandDef(
             id=0x01,
@@ -796,7 +764,6 @@ class Scenes(Cluster):
                 "transition_time?": t.uint16_t,
                 "scene_name?": t.LimitedCharString(16),
             },
-            direction=Direction.Server_to_Client,
         )
         # TODO: + extension field sets
         remove_scene_response: Final = ZCLCommandDef(
@@ -806,12 +773,10 @@ class Scenes(Cluster):
                 "group_id": t.Group,
                 "scene_id": t.uint8_t,
             },
-            direction=Direction.Server_to_Client,
         )
         remove_all_scenes_response: Final = ZCLCommandDef(
             id=0x03,
             schema={"status": foundation.Status, "group_id": t.Group},
-            direction=Direction.Server_to_Client,
         )
         store_scene_response: Final = ZCLCommandDef(
             id=0x04,
@@ -820,7 +785,6 @@ class Scenes(Cluster):
                 "group_id": t.Group,
                 "scene_id": t.uint8_t,
             },
-            direction=Direction.Server_to_Client,
         )
         get_scene_membership_response: Final = ZCLCommandDef(
             id=0x06,
@@ -830,7 +794,6 @@ class Scenes(Cluster):
                 "group_id": t.Group,
                 "scenes?": t.LVList[t.uint8_t],
             },
-            direction=Direction.Server_to_Client,
         )
         enhanced_add_response: Final = ZCLCommandDef(
             id=0x40,
@@ -839,7 +802,6 @@ class Scenes(Cluster):
                 "group_id": t.Group,
                 "scene_id": t.uint8_t,
             },
-            direction=Direction.Server_to_Client,
         )
         enhanced_view_response: Final = ZCLCommandDef(
             id=0x41,
@@ -850,7 +812,6 @@ class Scenes(Cluster):
                 "transition_time?": t.uint16_t,
                 "scene_name?": t.LimitedCharString(16),
             },
-            direction=Direction.Server_to_Client,
         )
         # TODO: + extension field sets
         copy_response: Final = ZCLCommandDef(
@@ -860,7 +821,6 @@ class Scenes(Cluster):
                 "group_id": t.Group,
                 "scene_id": t.uint8_t,
             },
-            direction=Direction.Server_to_Client,
         )
 
 
@@ -915,23 +875,14 @@ class OnOff(Cluster):
         reporting_status: Final = foundation.ZCL_REPORTING_STATUS_ATTR
 
     class ServerCommandDefs(BaseCommandDefs):
-        off: Final = ZCLCommandDef(
-            id=0x00, schema={}, direction=Direction.Client_to_Server
-        )
-        on: Final = ZCLCommandDef(
-            id=0x01, schema={}, direction=Direction.Client_to_Server
-        )
-        toggle: Final = ZCLCommandDef(
-            id=0x02, schema={}, direction=Direction.Client_to_Server
-        )
+        off: Final = ZCLCommandDef(id=0x00, schema={})
+        on: Final = ZCLCommandDef(id=0x01, schema={})
+        toggle: Final = ZCLCommandDef(id=0x02, schema={})
         off_with_effect: Final = ZCLCommandDef(
             id=0x40,
             schema={"effect_id": OffEffectIdentifier, "effect_variant": t.uint8_t},
-            direction=Direction.Client_to_Server,
-        )
-        on_with_recall_global_scene: Final = ZCLCommandDef(
-            id=0x41, schema={}, direction=Direction.Client_to_Server
         )
+        on_with_recall_global_scene: Final = ZCLCommandDef(id=0x41, schema={})
         on_with_timed_off: Final = ZCLCommandDef(
             id=0x42,
             schema={
@@ -939,7 +890,6 @@ class OnOff(Cluster):
                 "on_time": t.uint16_t,
                 "off_wait_time": t.uint16_t,
             },
-            direction=Direction.Client_to_Server,
         )
 
 
@@ -1051,7 +1001,6 @@ class LevelControl(Cluster):
                 "options_mask?": OptionsMask,
                 "options_override?": Options,
             },
-            direction=Direction.Client_to_Server,
         )
         move: Final = ZCLCommandDef(
             id=0x01,
@@ -1061,7 +1010,6 @@ class LevelControl(Cluster):
                 "options_mask?": OptionsMask,
                 "options_override?": Options,
             },
-            direction=Direction.Client_to_Server,
         )
         step: Final = ZCLCommandDef(
             id=0x02,
@@ -1072,7 +1020,6 @@ class LevelControl(Cluster):
                 "options_mask?": OptionsMask,
                 "options_override?": Options,
             },
-            direction=Direction.Client_to_Server,
         )
         stop: Final = ZCLCommandDef(
             id=0x03,
@@ -1080,17 +1027,14 @@ class LevelControl(Cluster):
                 "options_mask?": OptionsMask,
                 "options_override?": Options,
             },
-            direction=Direction.Client_to_Server,
         )
         move_to_level_with_on_off: Final = ZCLCommandDef(
             id=0x04,
             schema={"level": t.uint8_t, "transition_time": t.uint16_t},
-            direction=Direction.Client_to_Server,
         )
         move_with_on_off: Final = ZCLCommandDef(
             id=0x05,
             schema={"move_mode": MoveMode, "rate": t.uint8_t},
-            direction=Direction.Client_to_Server,
         )
         step_with_on_off: Final = ZCLCommandDef(
             id=0x06,
@@ -1099,15 +1043,11 @@ class LevelControl(Cluster):
                 "step_size": t.uint8_t,
                 "transition_time": t.uint16_t,
             },
-            direction=Direction.Client_to_Server,
-        )
-        stop_with_on_off: Final = ZCLCommandDef(
-            id=0x07, schema={}, direction=Direction.Client_to_Server
         )
+        stop_with_on_off: Final = ZCLCommandDef(id=0x07, schema={})
         move_to_closest_frequency: Final = ZCLCommandDef(
             id=0x08,
             schema={"frequency": t.uint16_t},
-            direction=Direction.Client_to_Server,
         )
 
 
@@ -1128,24 +1068,16 @@ class Alarms(Cluster):
         reset_alarm: Final = ZCLCommandDef(
             id=0x00,
             schema={"alarm_code": t.uint8_t, "cluster_id": t.uint16_t},
-            direction=Direction.Client_to_Server,
-        )
-        reset_all_alarms: Final = ZCLCommandDef(
-            id=0x01, schema={}, direction=Direction.Client_to_Server
-        )
-        get_alarm: Final = ZCLCommandDef(
-            id=0x02, schema={}, direction=Direction.Client_to_Server
-        )
-        reset_alarm_log: Final = ZCLCommandDef(
-            id=0x03, schema={}, direction=Direction.Client_to_Server
         )
+        reset_all_alarms: Final = ZCLCommandDef(id=0x01, schema={})
+        get_alarm: Final = ZCLCommandDef(id=0x02, schema={})
+        reset_alarm_log: Final = ZCLCommandDef(id=0x03, schema={})
         # 0x04: ("publish_event_log", {}, False),
 
     class ClientCommandDefs(BaseCommandDefs):
         alarm: Final = ZCLCommandDef(
             id=0x00,
             schema={"alarm_code": t.uint8_t, "cluster_id": t.uint16_t},
-            direction=Direction.Client_to_Server,
         )
         get_alarm_response: Final = ZCLCommandDef(
             id=0x01,
@@ -1155,7 +1087,6 @@ class Alarms(Cluster):
                 "cluster_id?": t.uint16_t,
                 "timestamp?": t.uint32_t,
             },
-            direction=Direction.Server_to_Client,
         )
         # 0x02: ("get_event_log", {}, False),
 
@@ -1300,7 +1231,6 @@ class RSSILocation(Cluster):
                 "power": t.int16s,
                 "path_loss_exponent": t.uint16_t,
             },
-            direction=Direction.Client_to_Server,
         )
         set_dev_config: Final = ZCLCommandDef(
             id=0x01,
@@ -1311,12 +1241,10 @@ class RSSILocation(Cluster):
                 "num_rssi_measurements": t.uint8_t,
                 "reporting_period": t.uint16_t,
             },
-            direction=Direction.Client_to_Server,
         )
         get_dev_config: Final = ZCLCommandDef(
             id=0x02,
             schema={"target_addr": t.EUI64},
-            direction=Direction.Client_to_Server,
         )
         get_location_data: Final = ZCLCommandDef(
             id=0x03,
@@ -1325,7 +1253,6 @@ class RSSILocation(Cluster):
                 "num_responses": t.uint8_t,
                 "target_addr": t.EUI64,
             },
-            direction=Direction.Client_to_Server,
         )
         rssi_response: Final = ZCLCommandDef(
             id=0x04,
@@ -1337,7 +1264,6 @@ class RSSILocation(Cluster):
                 "rssi": t.int8s,
                 "num_rssi_measurements": t.uint8_t,
             },
-            direction=Direction.Server_to_Client,
         )
         send_pings: Final = ZCLCommandDef(
             id=0x05,
@@ -1346,7 +1272,6 @@ class RSSILocation(Cluster):
                 "num_rssi_measurements": t.uint8_t,
                 "calculation_period": t.uint16_t,
             },
-            direction=Direction.Client_to_Server,
         )
         anchor_node_announce: Final = ZCLCommandDef(
             id=0x06,
@@ -1356,7 +1281,6 @@ class RSSILocation(Cluster):
                 "y": t.int16s,
                 "z": t.int16s,
             },
-            direction=Direction.Client_to_Server,
         )
 
     class ClientCommandDefs(BaseCommandDefs):
@@ -1370,7 +1294,6 @@ class RSSILocation(Cluster):
                 "num_rssi_measurements?": t.uint8_t,
                 "reporting_period?": t.uint16_t,
             },
-            direction=Direction.Server_to_Client,
         )
         location_data_response: Final = ZCLCommandDef(
             id=0x01,
@@ -1386,34 +1309,24 @@ class RSSILocation(Cluster):
                 "quality_measure?": t.uint8_t,
                 "location_age?": t.uint16_t,
             },
-            direction=Direction.Server_to_Client,
-        )
-        location_data_notification: Final = ZCLCommandDef(
-            id=0x02, schema={}, direction=Direction.Client_to_Server
-        )
-        compact_location_data_notification: Final = ZCLCommandDef(
-            id=0x03, schema={}, direction=Direction.Client_to_Server
         )
+        location_data_notification: Final = ZCLCommandDef(id=0x02, schema={})
+        compact_location_data_notification: Final = ZCLCommandDef(id=0x03, schema={})
         rssi_ping: Final = ZCLCommandDef(
             id=0x04,
             schema={"location_type": t.uint8_t},
-            direction=Direction.Client_to_Server,
-        )
-        rssi_req: Final = ZCLCommandDef(
-            id=0x05, schema={}, direction=Direction.Client_to_Server
         )
+        rssi_req: Final = ZCLCommandDef(id=0x05, schema={})
         report_rssi_measurements: Final = ZCLCommandDef(
             id=0x06,
             schema={
                 "measuring_device": t.EUI64,
                 "neighbors": t.LVList[NeighborInfo],
             },
-            direction=Direction.Client_to_Server,
         )
         request_own_location: Final = ZCLCommandDef(
             id=0x07,
             schema={"ieee_of_blind_node": t.EUI64},
-            direction=Direction.Client_to_Server,
         )
 
 
@@ -1607,15 +1520,9 @@ class BinaryOutput(Cluster):
         relinquish_default: Final = ZCLAttributeDef(
             id=0x0068, type=t.Bool, access="r*w"
         )
-        resolution: Final = ZCLAttributeDef(
-            id=0x006A, type=t.Single, access="r"
-        )  # Does not seem to be in binary_output
         status_flags: Final = ZCLAttributeDef(
             id=0x006F, type=t.bitmap8, access="r", mandatory=True
         )
-        engineering_units: Final = ZCLAttributeDef(
-            id=0x0075, type=t.enum16, access="r"
-        )  # Does not seem to be in binary_output
         application_type: Final = ZCLAttributeDef(
             id=0x0100, type=t.uint32_t, access="r"
         )
@@ -1683,7 +1590,7 @@ class MultistateInput(Cluster):
             id=0x0051, type=t.Bool, access="r*w", mandatory=True
         )
         present_value: Final = ZCLAttributeDef(
-            id=0x0055, type=t.Single, access="r*w", mandatory=True
+            id=0x0055, type=t.uint16_t, access="r*w", mandatory=True
         )
         # 0x0057: ('priority_array', TODO.array),  # Array of 16 structures of (boolean,
         # single precision)
@@ -1716,13 +1623,13 @@ class MultistateOutput(Cluster):
             id=0x0051, type=t.Bool, access="r*w", mandatory=True
         )
         present_value: Final = ZCLAttributeDef(
-            id=0x0055, type=t.Single, access="r*w", mandatory=True
+            id=0x0055, type=t.uint16_t, access="r*w", mandatory=True
         )
         # 0x0057: ZCLAttributeDef('priority_array', type=TODO.array),  # Array of 16 structures of (boolean,
         # single precision)
         reliability: Final = ZCLAttributeDef(id=0x0067, type=t.enum8, access="r*w")
         relinquish_default: Final = ZCLAttributeDef(
-            id=0x0068, type=t.Single, access="r*w"
+            id=0x0068, type=t.uint16_t, access="r*w"
         )
         status_flags: Final = ZCLAttributeDef(
             id=0x006F, type=t.bitmap8, access="r", mandatory=True
@@ -1752,13 +1659,13 @@ class MultistateValue(Cluster):
             id=0x0051, type=t.Bool, access="r*w", mandatory=True
         )
         present_value: Final = ZCLAttributeDef(
-            id=0x0055, type=t.Single, access="r*w", mandatory=True
+            id=0x0055, type=t.uint16_t, access="r*w", mandatory=True
         )
         # 0x0057: ZCLAttributeDef('priority_array', type=TODO.array),  # Array of 16 structures of (boolean,
         # single precision)
         reliability: Final = ZCLAttributeDef(id=0x0067, type=t.enum8, access="r*w")
         relinquish_default: Final = ZCLAttributeDef(
-            id=0x0068, type=t.Single, access="r*w"
+            id=0x0068, type=t.uint16_t, access="r*w"
         )
         status_flags: Final = ZCLAttributeDef(
             id=0x006F, type=t.bitmap8, access="r", mandatory=True
@@ -1872,44 +1779,36 @@ class Commissioning(Cluster):
         restart_device: Final = ZCLCommandDef(
             id=0x00,
             schema={"options": t.bitmap8, "delay": t.uint8_t, "jitter": t.uint8_t},
-            direction=Direction.Client_to_Server,
         )
         save_startup_parameters: Final = ZCLCommandDef(
             id=0x01,
             schema={"options": t.bitmap8, "index": t.uint8_t},
-            direction=Direction.Client_to_Server,
         )
         restore_startup_parameters: Final = ZCLCommandDef(
             id=0x02,
             schema={"options": t.bitmap8, "index": t.uint8_t},
-            direction=Direction.Client_to_Server,
         )
         reset_startup_parameters: Final = ZCLCommandDef(
             id=0x03,
             schema={"options": t.bitmap8, "index": t.uint8_t},
-            direction=Direction.Client_to_Server,
         )
 
     class ClientCommandDefs(BaseCommandDefs):
         restart_device_response: Final = ZCLCommandDef(
             id=0x00,
             schema={"status": foundation.Status},
-            direction=Direction.Server_to_Client,
         )
         save_startup_params_response: Final = ZCLCommandDef(
             id=0x01,
             schema={"status": foundation.Status},
-            direction=Direction.Server_to_Client,
         )
         restore_startup_params_response: Final = ZCLCommandDef(
             id=0x02,
             schema={"status": foundation.Status},
-            direction=Direction.Server_to_Client,
         )
         reset_startup_params_response: Final = ZCLCommandDef(
             id=0x03,
             schema={"status": foundation.Status},
-            direction=Direction.Server_to_Client,
         )
 
 
@@ -1978,71 +1877,89 @@ class UpgradeTimeoutPolicy(t.enum8):
     Do_not_apply_after_timeout = 0x01
 
 
-class ImageNotifyCommand(foundation.CommandSchema):
-    class PayloadType(t.enum8):
-        QueryJitter = 0x00
-        QueryJitter_ManufacturerCode = 0x01
-        QueryJitter_ManufacturerCode_ImageType = 0x02
-        QueryJitter_ManufacturerCode_ImageType_NewFileVersion = 0x03
+class ImageNotifyPayloadType(t.enum8):
+    QueryJitter = 0x00
+    QueryJitter_ManufacturerCode = 0x01
+    QueryJitter_ManufacturerCode_ImageType = 0x02
+    QueryJitter_ManufacturerCode_ImageType_NewFileVersion = 0x03
 
-    payload_type: None = t.StructField(type=PayloadType)
+
+class ImageNotifyCommand(foundation.CommandSchema):
+    PayloadType = ImageNotifyPayloadType
+
+    payload_type: ImageNotifyPayloadType
     query_jitter: t.uint8_t
     manufacturer_code: t.uint16_t = t.StructField(
         requires=(
-            lambda s: s.payload_type >= s.PayloadType.QueryJitter_ManufacturerCode
+            lambda s: s.payload_type
+            >= ImageNotifyPayloadType.QueryJitter_ManufacturerCode
         )
     )
     image_type: t.uint16_t = t.StructField(
         requires=(
             lambda s: s.payload_type
-            >= s.PayloadType.QueryJitter_ManufacturerCode_ImageType
+            >= ImageNotifyPayloadType.QueryJitter_ManufacturerCode_ImageType
         )
     )
     new_file_version: t.uint32_t = t.StructField(
         requires=(
             lambda s: s.payload_type
-            >= s.PayloadType.QueryJitter_ManufacturerCode_ImageType_NewFileVersion
+            >= ImageNotifyPayloadType.QueryJitter_ManufacturerCode_ImageType_NewFileVersion
         )
     )
 
 
-class QueryNextImageCommand(foundation.CommandSchema):
-    class FieldControl(t.bitmap8):
-        HardwareVersion = 0b00000001
+class QueryNextImageCommandFieldControl(t.bitmap8):
+    HardwareVersion = 0b00000001
 
-    field_control: None = t.StructField(type=FieldControl)
+
+class QueryNextImageCommand(foundation.CommandSchema):
+    FieldControl = QueryNextImageCommandFieldControl
+
+    field_control: QueryNextImageCommandFieldControl
     manufacturer_code: t.uint16_t
     image_type: t.uint16_t
     current_file_version: t.uint32_t
     hardware_version: t.uint16_t = t.StructField(
-        requires=(lambda s: s.field_control & s.FieldControl.HardwareVersion)
+        requires=(
+            lambda s: s.field_control
+            & QueryNextImageCommandFieldControl.HardwareVersion
+        )
     )
 
 
-class ImageBlockCommand(foundation.CommandSchema):
-    class FieldControl(t.bitmap8):
-        RequestNodeAddr = 0b00000001
-        MinimumBlockPeriod = 0b00000010
+class ImageBlockCommandFieldControl(t.bitmap8):
+    RequestNodeAddr = 0b00000001
+    MinimumBlockPeriod = 0b00000010
 
-    field_control: None = t.StructField(type=FieldControl)
+
+class ImageBlockCommand(foundation.CommandSchema):
+    FieldControl = ImageBlockCommandFieldControl
+
+    field_control: ImageBlockCommandFieldControl
     manufacturer_code: t.uint16_t
     image_type: t.uint16_t
     file_version: t.uint32_t
     file_offset: t.uint32_t
     maximum_data_size: t.uint8_t
     request_node_addr: t.EUI64 = t.StructField(
-        requires=(lambda s: s.field_control & s.FieldControl.RequestNodeAddr)
+        requires=(
+            lambda s: s.field_control & ImageBlockCommandFieldControl.RequestNodeAddr
+        )
     )
     minimum_block_period: t.uint16_t = t.StructField(
-        requires=(lambda s: s.field_control & s.FieldControl.MinimumBlockPeriod)
+        requires=(
+            lambda s: s.field_control & ImageBlockCommandFieldControl.MinimumBlockPeriod
+        )
     )
 
 
+class ImagePageCommandFieldControl(t.bitmap8):
+    RequestNodeAddr = 0b00000001
+
+
 class ImagePageCommand(foundation.CommandSchema):
-    class FieldControl(t.bitmap8):
-        RequestNodeAddr = 0b00000001
-
-    field_control: None = t.StructField(type=FieldControl)
+    field_control: ImagePageCommandFieldControl
     manufacturer_code: t.uint16_t
     image_type: t.uint16_t
     file_version: t.uint32_t
@@ -2051,7 +1968,9 @@ class ImagePageCommand(foundation.CommandSchema):
     page_size: t.uint16_t
     response_spacing: t.uint16_t
     request_node_addr: t.EUI64 = t.StructField(
-        requires=lambda s: s.field_control & s.FieldControl.RequestNodeAddr
+        requires=lambda s: (
+            s.field_control & ImagePageCommandFieldControl.RequestNodeAddr
+        )
     )
 
 
@@ -2137,15 +2056,9 @@ class Ota(Cluster):
         reporting_status: Final = foundation.ZCL_REPORTING_STATUS_ATTR
 
     class ServerCommandDefs(BaseCommandDefs):
-        query_next_image: Final = ZCLCommandDef(
-            id=0x01, schema=QueryNextImageCommand, direction=Direction.Client_to_Server
-        )
-        image_block: Final = ZCLCommandDef(
-            id=0x03, schema=ImageBlockCommand, direction=Direction.Client_to_Server
-        )
-        image_page: Final = ZCLCommandDef(
-            id=0x04, schema=ImagePageCommand, direction=Direction.Client_to_Server
-        )
+        query_next_image: Final = ZCLCommandDef(id=0x01, schema=QueryNextImageCommand)
+        image_block: Final = ZCLCommandDef(id=0x03, schema=ImageBlockCommand)
+        image_page: Final = ZCLCommandDef(id=0x04, schema=ImagePageCommand)
         upgrade_end: Final = ZCLCommandDef(
             id=0x06,
             schema={
@@ -2154,7 +2067,6 @@ class Ota(Cluster):
                 "image_type": t.uint16_t,
                 "file_version": t.uint32_t,
             },
-            direction=Direction.Client_to_Server,
         )
         query_specific_file: Final = ZCLCommandDef(
             id=0x08,
@@ -2165,13 +2077,10 @@ class Ota(Cluster):
                 "file_version": t.uint32_t,
                 "current_zigbee_stack_version": t.uint16_t,
             },
-            direction=Direction.Client_to_Server,
         )
 
     class ClientCommandDefs(BaseCommandDefs):
-        image_notify: Final = ZCLCommandDef(
-            id=0x00, schema=ImageNotifyCommand, direction=Direction.Client_to_Server
-        )
+        image_notify: Final = ZCLCommandDef(id=0x00, schema=ImageNotifyCommand)
         query_next_image_response: Final = ZCLCommandDef(
             id=0x02,
             schema={
@@ -2181,12 +2090,10 @@ class Ota(Cluster):
                 "file_version?": t.uint32_t,
                 "image_size?": t.uint32_t,
             },
-            direction=Direction.Server_to_Client,
         )
         image_block_response: Final = ZCLCommandDef(
             id=0x05,
             schema=ImageBlockResponseCommand,
-            direction=Direction.Server_to_Client,
         )
         upgrade_end_response: Final = ZCLCommandDef(
             id=0x07,
@@ -2197,7 +2104,6 @@ class Ota(Cluster):
                 "current_time": t.UTCTime,
                 "upgrade_time": t.UTCTime,
             },
-            direction=Direction.Server_to_Client,
         )
         query_specific_file_response: Final = ZCLCommandDef(
             id=0x09,
@@ -2208,7 +2114,6 @@ class Ota(Cluster):
                 "file_version?": t.uint32_t,
                 "image_size?": t.uint32_t,
             },
-            direction=Direction.Server_to_Client,
         )
 
     def handle_cluster_request(
@@ -2310,11 +2215,8 @@ class PowerProfile(Cluster):
         power_profile_request: Final = ZCLCommandDef(
             id=0x00,
             schema={"power_profile_id": t.uint8_t},
-            direction=Direction.Client_to_Server,
-        )
-        power_profile_state_request: Final = ZCLCommandDef(
-            id=0x01, schema={}, direction=Direction.Client_to_Server
         )
+        power_profile_state_request: Final = ZCLCommandDef(id=0x01, schema={})
         get_power_profile_price_response: Final = ZCLCommandDef(
             id=0x02,
             schema={
@@ -2323,7 +2225,6 @@ class PowerProfile(Cluster):
                 "price": t.uint32_t,
                 "price_trailing_digit": t.uint8_t,
             },
-            direction=Direction.Server_to_Client,
         )
         get_overall_schedule_price_response: Final = ZCLCommandDef(
             id=0x03,
@@ -2332,7 +2233,6 @@ class PowerProfile(Cluster):
                 "price": t.uint32_t,
                 "price_trailing_digit": t.uint8_t,
             },
-            direction=Direction.Server_to_Client,
         )
         energy_phases_schedule_notification: Final = ZCLCommandDef(
             id=0x04,
@@ -2340,7 +2240,6 @@ class PowerProfile(Cluster):
                 "power_profile_id": t.uint8_t,
                 "scheduled_phases": t.LVList[ScheduleRecord],
             },
-            direction=Direction.Client_to_Server,
         )
         energy_phases_schedule_response: Final = ZCLCommandDef(
             id=0x05,
@@ -2348,17 +2247,14 @@ class PowerProfile(Cluster):
                 "power_profile_id": t.uint8_t,
                 "scheduled_phases": t.LVList[ScheduleRecord],
             },
-            direction=Direction.Server_to_Client,
         )
         power_profile_schedule_constraints_request: Final = ZCLCommandDef(
             id=0x06,
             schema={"power_profile_id": t.uint8_t},
-            direction=Direction.Client_to_Server,
         )
         energy_phases_schedule_state_request: Final = ZCLCommandDef(
             id=0x07,
             schema={"power_profile_id": t.uint8_t},
-            direction=Direction.Client_to_Server,
         )
         get_power_profile_price_extended_response: Final = ZCLCommandDef(
             id=0x08,
@@ -2368,7 +2264,6 @@ class PowerProfile(Cluster):
                 "price": t.uint32_t,
                 "price_trailing_digit": t.uint8_t,
             },
-            direction=Direction.Server_to_Client,
         )
 
     class ClientCommandDefs(BaseCommandDefs):
@@ -2379,7 +2274,6 @@ class PowerProfile(Cluster):
                 "power_profile_id": t.uint8_t,
                 "transfer_phases": t.LVList[PowerProfilePhase],
             },
-            direction=Direction.Client_to_Server,
         )
         power_profile_response: Final = ZCLCommandDef(
             id=0x01,
@@ -2388,30 +2282,23 @@ class PowerProfile(Cluster):
                 "power_profile_id": t.uint8_t,
                 "transfer_phases": t.LVList[PowerProfilePhase],
             },
-            direction=Direction.Server_to_Client,
         )
         power_profile_state_response: Final = ZCLCommandDef(
             id=0x02,
             schema={"power_profiles": t.LVList[PowerProfileType]},
-            direction=Direction.Server_to_Client,
         )
         get_power_profile_price: Final = ZCLCommandDef(
             id=0x03,
             schema={"power_profile_id": t.uint8_t},
-            direction=Direction.Client_to_Server,
         )
         power_profile_state_notification: Final = ZCLCommandDef(
             id=0x04,
             schema={"power_profiles": t.LVList[PowerProfileType]},
-            direction=Direction.Client_to_Server,
-        )
-        get_overall_schedule_price: Final = ZCLCommandDef(
-            id=0x05, schema={}, direction=Direction.Client_to_Server
         )
+        get_overall_schedule_price: Final = ZCLCommandDef(id=0x05, schema={})
         energy_phases_schedule_request: Final = ZCLCommandDef(
             id=0x06,
             schema={"power_profile_id": t.uint8_t},
-            direction=Direction.Client_to_Server,
         )
         energy_phases_schedule_state_response: Final = ZCLCommandDef(
             id=0x07,
@@ -2419,7 +2306,6 @@ class PowerProfile(Cluster):
                 "power_profile_id": t.uint8_t,
                 "num_scheduled_energy_phases": t.uint8_t,
             },
-            direction=Direction.Server_to_Client,
         )
         energy_phases_schedule_state_notification: Final = ZCLCommandDef(
             id=0x08,
@@ -2427,7 +2313,6 @@ class PowerProfile(Cluster):
                 "power_profile_id": t.uint8_t,
                 "num_scheduled_energy_phases": t.uint8_t,
             },
-            direction=Direction.Client_to_Server,
         )
         power_profile_schedule_constraints_notification: Final = ZCLCommandDef(
             id=0x09,
@@ -2436,7 +2321,6 @@ class PowerProfile(Cluster):
                 "start_after": t.uint16_t,
                 "stop_before": t.uint16_t,
             },
-            direction=Direction.Client_to_Server,
         )
         power_profile_schedule_constraints_response: Final = ZCLCommandDef(
             id=0x0A,
@@ -2445,7 +2329,6 @@ class PowerProfile(Cluster):
                 "start_after": t.uint16_t,
                 "stop_before": t.uint16_t,
             },
-            direction=Direction.Server_to_Client,
         )
         get_power_profile_price_extended: Final = ZCLCommandDef(
             id=0x0B,
@@ -2454,7 +2337,6 @@ class PowerProfile(Cluster):
                 "power_profile_id": t.uint8_t,
                 "power_profile_start_time?": t.uint16_t,
             },
-            direction=Direction.Client_to_Server,
         )
 
 
@@ -2508,26 +2390,19 @@ class PollControl(Cluster):
         checkin_response: Final = ZCLCommandDef(
             id=0x00,
             schema={"start_fast_polling": t.Bool, "fast_poll_timeout": t.uint16_t},
-            direction=Direction.Server_to_Client,
-        )
-        fast_poll_stop: Final = ZCLCommandDef(
-            id=0x01, schema={}, direction=Direction.Client_to_Server
         )
+        fast_poll_stop: Final = ZCLCommandDef(id=0x01, schema={})
         set_long_poll_interval: Final = ZCLCommandDef(
             id=0x02,
             schema={"new_long_poll_interval": t.uint32_t},
-            direction=Direction.Client_to_Server,
         )
         set_short_poll_interval: Final = ZCLCommandDef(
             id=0x03,
             schema={"new_short_poll_interval": t.uint16_t},
-            direction=Direction.Client_to_Server,
         )
 
     class ClientCommandDefs(BaseCommandDefs):
-        checkin: Final = ZCLCommandDef(
-            id=0x0000, schema={}, direction=Direction.Client_to_Server
-        )
+        checkin: Final = ZCLCommandDef(id=0x0000, schema={})
 
 
 class GreenPowerProxy(Cluster):
